/*                                                                -*- mode: c; c-basic-offset: 8;  -*-
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/*
 ============================================================================
 All definitions in one file
 ============================================================================
 */



#ifndef DEFINITIONS_H
#define DEFINITIONS_H
	
/**********************************************************************
 * Glue 1.3 Constants
 */

/* LDAP attributes */
#define ATTR_OC			"objectClass"
#define ATTR_DN			"dn"
#define ATTR_NAME		"GlueServiceUniqueID"
#define ATTR_TYPE		"GlueServiceType"
#define ATTR_VERSION		"GlueServiceVersion"
#define ATTR_LINK		"GlueForeignKey"
#define ATTR_SITE		"GlueSiteUniqueID"
#define ATTR_ENDPOINT		"GlueServiceEndpoint"
#define ATTR_VERSION		"GlueServiceVersion"
#define ATTR_STATUS     	"GlueServiceStatus"
#define ATTR_WSDL		"GlueServiceWSDL"
#define ATTR_ADMIN
#define ATTR_VO			"GlueServiceOwner"
#define ATTR_DATAKEY		"GlueServiceDataKey"
#define ATTR_DATAVALUE		"GlueServiceDataValue"

/* in gfal used attributes*/
#define ATTR_CE_AP_ATNM		"GlueCESEBindCEAccesspoint"
#define ATTR_PROTO_PORT 	"GlueSEAccessProtocolPort"
#define	ATTR_PROTO_TYPE		"GlueSEAccessProtocolType"
#define ATTR_SA_PATH_ATNM 	"GlueSAPath"
#define ATTR_SA_ROOT_ATNM	"GlueSARoot"
#define ATTR_PATH_ATNM 		"GlueVOInfoPath"
#define ATTR_SA_KEY_ATNM 	"GlueChunkKey"
#define ATTR_SE_TYPE_ATPT	"GlueSEPort"
#define ATTR_SE_TYPE_ATST	"GlueSEStatus"

/* nonstandard glue stuff to fix LCG inconsistencies */
#define ATTR_VERSION_MAJOR	"GlueServiceMajorVersion"
#define ATTR_VERSION_MINOR	"GlueServiceMinorVersion"
#define ATTR_VERSION_PATCH	"GlueServicePatchVersion"
#define ATTR_URI		"GlueServiceURI"
#define ATTR_NM			"GlueServiceName"

#define ATTR_ACCESSRULE     	"GlueServiceAccessControlRule"
#define ATTR_ACCESSRULEBASE	"GlueServiceAccessControlBaseRule"

#define ATTR_HOSTINGORG     	"GlueServiceHostingOrganization"
#define ATTR_SA_ROOT        	"GlueSARoot"
#define ATTR_SA_ACBR        	"GlueSAAccessControlBaseRule"
#define ATTR_SE	        	"GlueSEUniqueID"
#define ATTR_CHUNK		"GlueChunkKey"

/* LDAP object classes */
#define CLASS_SERVICE		"GlueService"
#define CLASS_DATA		"GlueServiceData"
#define CLASS_KEY		"GlueKey"
#define CLASS_SA		"GlueSA"

/* LDAP filter definitions */
/*  for gfal */

#define QUERY_RLS "(&(GlueServiceType=edg-*)(|(GlueServiceAccessControlBaseRule=%s)(GlueServiceAccessControlRule=%s)))"
#define QUERY_TYPE_ENDPOINTS	"(|(GlueSEUniqueID=*%s*)(&(GlueServiceType=srm*)(GlueServiceEndpoint=*://%s:%s*)))"

#define QUERY_LFC_ENDPOINT 	"(&(GlueServiceType=lcg-file-catalog)%s)"

#define QUERY_SE_STATUS 	"(&(GlueSEStatus=*Production*)(&(GlueSEUniqueID=*%s*)))"

#define QUERY_SE_IMPL 	"(&(GlueSEImplementationName=*dcache*)(&(GlueSEUniqueID=*%s*)))"

#define QUERY_CESE_BIND "(GlueCESEBindSEUniqueID=%s)"

#define QUERY_PROTO_PORT_TYPE "(&(GlueSEAccessProtocolType=*)(GlueChunkKey=GlueSEUniqueID=*%s*))"

#define QUERY_CE_AP_ATNM 	"(" ATTR_CE_AP_ATNM "=%s)"

#define QUERY_SA_PATH	"(&(GlueSALocalID=%s)(" ATTR_CHUNK "=" ATTR_SE "=%s))"

#define QUERY_NAME	"(&" \
					"(" ATTR_OC "=" CLASS_SERVICE ")" \
					"(|(" ATTR_NAME "=*%s*)" \
					"(" ATTR_URI "=*%s*))" \
				    ")"

#define QUERY_DATA	"(" ATTR_OC "=" CLASS_DATA ")"

#define QUERY_DATAKEY "(&" \
					    QUERY_DATA \
					  "(" ATTR_DATAKEY "=%s)" \
					  ")"

/* Query fragments for listServicesByData() */
#define QUERY_OC_SERVICE "(" ATTR_OC "=" CLASS_SERVICE ")"
#define QUERY_OC_SA		 "(" ATTR_OC "=" CLASS_SA ")"
#define QUERY_SITE		 "(&" \
					"(" ATTR_OC "=" CLASS_KEY ")" \
					"(" ATTR_LINK "=" ATTR_SITE "=%s)" \
				    ")"
#define QUERY_TYPE "(" ATTR_TYPE "=%s)"

#define QUERY_SRM_TYPE	"(|" \
					"(" ATTR_TYPE "=SRM)" \
					"(" ATTR_TYPE "=srm_v1)" \
				    ")"


#define QUERY_GSIFTP_TYPE	"(|" \
					"(" ATTR_TYPE "=gsiftp)" \
					"(" ATTR_TYPE "=gsiftp)" \
				    ")"

#define QUERY_HOST "(|(" ATTR_ENDPOINT "=*%s*)" \
                   "(" ATTR_URI  "=*%s*))"


#define QUERY_SE "(" ATTR_CHUNK "=" ATTR_SE "=%s)"


/* Query expressing "No VO attribute" */
#define QUERY_VO_ANY "(!(" ATTR_VO "=*))"

#define QUERY_VO_PRE  "(|" QUERY_VO_ANY
#define QUERY_VO      "(" ATTR_VO "=%s)"
#define QUERY_VO_POST ")"


#define MAX_URI_LENGTH 1024

/* LDAP query timeout */
#define QUERY_TIMEOUT  60;

#define ERROR_OUTOFMEMORY  "Insufficient memory to complete API call."
#define ERROR_NOPLUGIN     "Service discovery plugin was not found."
#define ERROR_NULLPLUGIN   "Plugin cannot be NULL."
#define ERROR_NOPLUGINNAME "Plugin name is missing."
#define ERROR_PLUGINEXISTS "Plugin is already registered."
#define ERROR_BADPLUGIN    "The plugin does not implement a mandatory method."


/**********************************************************************
 * Global variables
 */
extern char *SD_EMPTY_MESSAGE; /* default error message */
extern char *SD_NOMEM_MESSAGE; /* error message when there were not enough*/


/* Data key object attributes */

extern char *data_attrs[];

extern char *all_attrs[];

extern char *no_attrs[];

extern char *link_attrs[];

extern char *service_attrs[];

extern char *site_attrs[];

extern char *sa_attrs[];

extern char *wsdl_attrs[];

extern char *sa_path_attrs[];

extern char *lfc_endpoint_attrs[];

extern char *ce_ap_attrs[];

extern char *seap_attrs[];

extern char* available_plungins[];




/**********************************************************************
 * Glue2 Constants - gfal only
 */

/* LDAP attributes */
#define GLUE2_ATTR_ENDPOINT		"GLUE2EndpointURL"
#define GLUE2_ATTR_TYPE			"GLUE2EndpointInterfaceName"
#define GLUE2_ATTR_VERSION		"GLUE2EndpointInterfaceVersion"


/* in gfal used attributes*/
#define GLUE2_ATTR_CE_AP_ATNM		"GLUE2ToComputingServiceLocalPath"
#define  GLUE2_POLICY_ID		"GLUE2PolicyID"
#define  GLUE2_ACCESS_POLICY_FK		"GLUE2AccessPolicyEndpointForeignKey"
#define  GLUE2_POLICY_RULE		"GLUE2PolicyRule"
#define GLUE2_GLUE2ShareID		"GLUE2ShareID"
#define GLUE2_GLUE2StorageSharePath	"GLUE2StorageSharePath"
#define GLUE2EndpointServiceForeignKey "GLUE2EndpointServiceForeignKey"
#define GLUE2EndpointID			"GLUE2EndpointID"

/* LDAP filter definitions */
/* for gfal only */
#define GLUE2_QUERY_LFC_POLICY		"(&(objectClass=GLUE2AccessPolicy)(GLUE2AccessPolicyEndpointForeignKey=*-lfc-endpoint*)%s)"

#define GLUE2_QUERY_LFC_ENDPOINT 	"(&(objectClass=GLUE2Endpoint)(GLUE2EndpointInterfaceName=lcg-file-catalog)(GLUE2EndpointURL=*lfc://%s:*))"

#define GLUE2_QUERY_TYPE_ENDPOINTS	"(&(objectClass=GLUE2Endpoint)(GLUE2EndpointInterfaceName=SRM)(GLUE2EndpointServiceForeignKey=%s))"

#define GLUE2_QUERY_SE_STATUS 		"(&(objectClass=GLUE2Endpoint)(GLUE2EndpointHealthState=ok)(GLUE2EndpointQualityLevel=production)(GLUE2EndpointServiceForeignKey=%s))"

#define GLUE2_QUERY_PROTO_PORT_TYPE 	"(&(objectClass=GLUE2Endpoint)(GLUE2EndpointServiceForeignKey=%s))"

#define GLUE2_QUERY_CESE_BIND 		"(&(objectClass=GLUE2ToComputingService)(GLUE2ToComputingServiceComputingServiceForeignKey=%s))"

#define GLUE2_STORAGE_PATH		"(&(objectClass=GLUE2Share)(GLUE2ShareServiceForeignKey=%s)(GLUE2StorageShareTag=%s))"

#define GLUE2_STORAGE_PATH_NO_ST	"(&(objectClass=GLUE2Share)(GLUE2ShareServiceForeignKey=%s))"

#define GLUE2_STORAGE_POLICY		"(&(objectClass=GLUE2Policy)(GLUE2PolicyID=*%s*)(&(GLUE2PolicyRule=VO:%s)))"

#define GLUE2_SERVICE_ID		"(&(objectClass=GLUE2Endpoint)(GLUE2EndpointInterfaceName=SRM)(GLUE2EndpointURL=*://%s*))"


extern char *g2_storage_path_attrs[];

/* Data key object attributes */
/*must be null terminated*/

extern char *g2_lfc_endpoint_attrs[];

extern char *g2_lfc_endpoint_foreign_key_attrs[];

extern char *g2_lfc_policy_attrs[];

extern char *g2_ce_ap_attrs[];

extern char *g2_seap_attrs[];


typedef struct {
	char *GLUE2ShareID;
	char *GLUE2StorageSharePath;
        int numItems;

    } storages;

#endif
